﻿//////////////////////////////////////////////
// MeshUtils.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class DecodedMeshData ;
	class Mesh ;

	struct VertexComposition ;
	struct VertexData ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../../Bounds/BoundingBox.h"

#include "../../Dll/DllDefines.h"

#include "../InputLayouts/MeshInputLayoutAttribute.h"

#include "MeshFillOptions.h"
#include "AttributeGenerationInfo.h"
#include "AttributeGenerationOutput.h"
#include "PackedMeshData.h"

// nkMemory
#include <NilkinsMemory/Containers/BufferView.h>

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT MeshUtils final
	{
		private :

			// Constructor, destructor
			MeshUtils () = delete ;
			~MeshUtils () = delete ;

		public :

			// Translation
			static PackedMeshData packIntoMeshData (const nkMemory::BufferView<VertexData>& meshInfo, const VertexComposition& composition) ;

			// Filling
			static void fillMeshFromDecodedData (DecodedMeshData& data, Mesh* mesh, const MeshFillOptions& options = MeshFillOptions()) ;

			// Computing
			static BoundingBox computeBounds (const nkMemory::BufferView<unsigned char>& vertexBuffer, unsigned int byteStride, const MeshInputLayoutAttribute& positionAttribute) ;
			static AttributeGenerationOutput computeNormals (const AttributeGenerationInfo& inputInfo) ;
			static void computeNormals (const AttributeGenerationInfo& inputInfo, AttributeGenerationOutput& outputInfo) ;
			static AttributeGenerationOutput computeBinormals (const AttributeGenerationInfo& inputInfo) ;
			static void computeBinormals (const AttributeGenerationInfo& inputInfo, AttributeGenerationOutput& outputInfo) ;
			static AttributeGenerationOutput computeTangents (const AttributeGenerationInfo& inputInfo) ;
			static void computeTangents (const AttributeGenerationInfo& inputInfo, AttributeGenerationOutput& outputInfo) ;
	} ;
}