﻿//////////////////////////////////////////////
// ResourceSetLoadResult.h
//
//////////////////////////////////////////////

/// Defines / macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class BlendState ;
	class Buffer ;
	class Camera ;
	class Compositor ;
	class DepthStencilState ;
	class Mesh ;
	class Node ;
	class Program ;
	class RasterState ;
	class RenderQueue ;
	class Sampler ;
	class Shader ;
	class Texture ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// Standards
#include <vector>
	
/// Classe -----------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT ResourceSetLoadResult final
	{
		private :
		
			// Attributs
			// Les modèles
			std::vector<Mesh*> _meshes ;
			// Les shaders
			std::vector<Shader*> _shaders ;
			// Les shader programs
			std::vector<Program*> _programs ;
			// Les samplers chargés
			std::vector<Sampler*> _samplers ;
			// Les textures
			std::vector<Texture*> _textures ;
			// Les scènes
			std::vector<Node*> _scenes ;
			// Les compositors
			std::vector<Compositor*> _compositors ;
			// Les compute buffers
			std::vector<Buffer*> _computeBuffers ;
			// States
			std::vector<BlendState*> _blendStates ;
			std::vector<DepthStencilState*> _depthStencilStates ;
			std::vector<RasterState*> _rasterStates ;
			// Caméras
			std::vector<Camera*> _cameras ;
			// RQ
			std::vector<RenderQueue*> _renderQueues ;

			// Compositor actif
			Compositor* _defaultCompositor ;
			
		public :
		
			// Constructeur, destructeur
			ResourceSetLoadResult () ;
			~ResourceSetLoadResult () ;

			// Getters / Setters
			std::vector<Mesh*>& getMeshes () ;
			std::vector<Shader*>& getShaders () ;
			std::vector<Program*>& getPrograms () ;
			std::vector<Sampler*>& getSamplers () ;
			std::vector<Texture*>& getTextures () ;
			std::vector<Node*>& getScenes () ;
			std::vector<Compositor*>& getCompositors () ;
			std::vector<Buffer*>& getBuffers () ;
			std::vector<BlendState*>& getBlendStates () ;
			std::vector<DepthStencilState*>& getDepthStencilStates () ;
			std::vector<RasterState*>& getRasterStates () ;
			std::vector<Camera*>& getCameras () ;
			std::vector<RenderQueue*>& getRenderQueues () ;
			Compositor* getDefaultCompositor () const ;

			void setDefaultCompositor (Compositor* value) ;

			// Utils
			void clear () ;
	} ;
}